import datetime
import hashlib
import json
import os
import tempfile
import unicodedata

import requests
from colorama import Fore, Style

from spyderml import __version__


class Cacherequest:

    def __init__(self, cache=True, life=1, proxy=None, headers=None):
        self.cache = cache
        self.life = life
        self.version = __version__.replace(".", "")
        self.text = None
        self.headers_text = []
        self.proxy = proxy
        self.headers = headers
        self.session = requests.Session()

    def get(self, url):
        path_to_cache_file = os.path.join(tempfile.gettempdir(), "get" +
                                          hashlib.md5(url.encode()).hexdigest() + self.version)
        if self.cache:
            if os.path.exists(path_to_cache_file):
                buffer = json.loads(open(path_to_cache_file, 'r').read())
                if datetime.datetime.utcnow() < datetime.datetime.strptime(buffer["data"], "%Y-%m-%d %H:%M:%S"):
                    self.text = buffer["html"]
                    return True

        try:
            if self.headers is not None:
                if self.proxy is not None:
                    self.session.proxies.update(self.proxy)
                page = self.session.get(url=url, headers=self.headers, allow_redirects=True)
            else:
                if self.proxy is not None:
                    self.session.proxies.update(self.proxy)
                page = self.session.get(url)
            page.encoding = "utf-8"
            self.text = unicodedata.normalize(u'NFKD', page.text).encode('ascii', 'ignore').decode("utf-8")
            if self.cache:
                with open(path_to_cache_file, '+w') as file:
                    file.write(json.dumps({"data": (datetime.datetime.utcnow()
                                                    + datetime.timedelta(minutes=self.life)).strftime(
                        "%Y-%m-%d %H:%M:%S"),
                        "html": self.text}))
        except requests.exceptions.SSLError:
            print(f"{Fore.RED}{url}\t<SSL Error>{Style.RESET_ALL}")
            exit()
        except requests.exceptions.MissingSchema:
            print(f"{Fore.RED}{url}:\nInvalid url\n http://?{Style.RESET_ALL}")
            exit()
        except requests.exceptions.InvalidSchema:
            print(f"{Fore.RED}{url} ERROR{Style.RESET_ALL}")
            exit()
        except requests.exceptions.ConnectionError:
            print(f"{Fore.RED}Attempt to connect to the host: {url} refused...\n"
                  f"Tip: Try to exchange https for http or http for https.{Style.RESET_ALL}")
            exit()

    def post(self, url, data):
        path_to_cache_file = os.path.join(tempfile.gettempdir(), "post" +
                                          hashlib.md5(url.encode()).hexdigest() + self.version)
        if self.cache:
            if os.path.exists(path_to_cache_file):
                buffer = json.loads(open(path_to_cache_file, 'r').read())
                if datetime.datetime.utcnow() < datetime.datetime.strptime(buffer["data"], "%Y-%m-%d %H:%M:%S"):
                    self.text = buffer["html"]
                    return True
        try:
            if self.headers is not None:
                if self.proxy is not None:
                    self.session.proxies.update(self.proxy)
                if data is not None:
                    page = self.session.post(url=url, headers=self.headers, data=data, allow_redirects=True)
                else:
                    page = self.session.post(url=url, headers=self.headers, allow_redirects=True)
            else:
                if self.proxy is not None:
                    self.session.proxies.update(self.proxy)
                if data:
                    page = self.session.post(url, data=data, allow_redirects=True)
                else:
                    page = self.session.post(url, allow_redirects=True)
            page.encoding = "utf-8"
            self.text = unicodedata.normalize(u'NFKD', page.text).encode('ascii', 'ignore').decode("utf-8")
            if self.cache:
                with open(path_to_cache_file, '+w') as file:
                    file.write(json.dumps({"data": (datetime.datetime.utcnow()
                                                    + datetime.timedelta(minutes=self.life)).strftime(
                        "%Y-%m-%d %H:%M:%S"),
                        "html": self.text}))
        except requests.exceptions.SSLError:
            print(f"{Fore.RED}{url}\t<SSL Error>{Style.RESET_ALL}")
            exit()
        except requests.exceptions.MissingSchema:
            print(f"{Fore.RED}{url}:\nInvalid url\n http://?{Style.RESET_ALL}")
            exit()
        except requests.exceptions.InvalidSchema:
            print(f"{Fore.RED}{url} ERROR{Style.RESET_ALL}")
            exit()
        except requests.exceptions.ConnectionError:
            print(f"{Fore.RED}Attempt to connect to the host: {url} refused...\n"
                  f"Tip: Try to exchange https for http or http for https.{Style.RESET_ALL}")
            exit()

    def response_and_request_headers(self, url):
        try:
            if self.proxy is not None:
                self.session.proxies.update(self.proxy)
            r_headers = self.session.get(url)
            request_headers = r_headers.request.headers
            self.headers_text.append(request_headers)
            self.headers_text.append(r_headers.headers)
        except requests.exceptions.SSLError:
            print(f"{Fore.RED}{url}\t<SSL Error>{Style.RESET_ALL}")
            exit()
        except requests.exceptions.MissingSchema:
            print(f"{Fore.RED}{url}:\nInvalid url\n http://?{Style.RESET_ALL}")
            exit()
        except requests.exceptions.InvalidSchema:
            print(f"{Fore.RED}{url} ERROR{Style.RESET_ALL}")
            exit()
        except requests.exceptions.ConnectionError:
            print(f"{Fore.RED}Attempt to connect to the host: {url} refused...\n"
                  f"Tip: Try to exchange https for http or http for https.{Style.RESET_ALL}")
            exit()



import logging
import json
from urllib.parse import urljoin

import requests
import urllib3
import webtech
from bs4 import BeautifulSoup as Bs
from bs4 import Comment
from colorama import Fore, Style

from spyderml.lib.file import save_output


def treat_objects(objects: str):
    if "," in objects:
        objects = objects.split(',')
        return objects
    return objects


def update_database():
    webtech.database.update_database(force=True)


def detect_technologies(url, filepath=None):
    webtech.database.update_database()
    # you can use options, same as from the command line
    wt = webtech.WebTech(options={'json': True})

    # scan a single website
    try:
        report = wt.start_from_url(url)
        techs = report['tech']
        if filepath is not None:
            save_output(filename=filepath, text=f"\nSite: {url} technologies:")
        print("Site: {} technologies:".format(url))

        for tech in techs:
            if filepath is not None:
                save_output(filename=filepath, text=f"Name: {tech['name']}\tVersion: {tech['version']}")
            print(f'''{tech['version']}
            ''')
        print('\n')
    except webtech.utils.ConnectionException:
        print("Connection error")


def soup_tags(document, object, file=None):
    html = Bs(document, 'html.parser')
    results = html.find_all(object)
    for result in results:
        if file is not None:
            save_output(filename=file, text=result)
        print(result)


def soup_comments(document, file=None):
    html = Bs(document, 'html.parser')
    comments = html.find_all(string=lambda text: isinstance(text, Comment))
    for comment in comments:
        if file is not None:
            save_output(filename=file, text=comment)
        print(comment)


def soup_attrs(document, object, file=None):
    html = Bs(document, 'html.parser')
    if type(object) == list:
        for o in object:
            for attribute in html.select(f"[{o}]"):
                if file is not None:
                    save_output(filename=file, text=attribute)
                print(attribute)
    else:
        for attribute in html.select(f"[{object}]"):
            if file is not None:
                save_output(filename=file, text=attribute)
            print(attribute)


def get_js(url, document, file=None):
    html = Bs(document, 'html.parser')
    for script in html.find_all("script"):
        if script.attrs.get("src"):
            script_url = urljoin(url, script.attrs.get("src"))
            if file is not None:
                save_output(filename=file, text=script_url)
            print(script_url)


def print_html(document, file=None):
    if file is not None:
        save_output(filename=file, text=document)
    print(document)


def json_response(document, file=None):
    json_content = json.loads(document)
    if file is not None:
        save_output(filename=file, text=json_content)
    print(json.dumps(json_content, indent=4))


def print_headers(headers_dict_list, filenane=None):
    if filenane is not None:
        for index, headers_dict in enumerate(headers_dict_list):
            if index == 0:
                save_output(filename=filenane, text="------------Request Headers-----------")
            elif index == 1:
                save_output(filename=filenane, text="------------Response Headers-----------")
            for key in headers_dict:
                save_output(filename=filenane, text=f"{key}: {headers_dict[key]}")
            save_output(filename=filenane, text="-"*39+"\n")
    for index, headers_dict in enumerate(headers_dict_list):
        if index == 0:
            print(f"------------{Fore.CYAN}Request Headers{Style.RESET_ALL}-----------")
        elif index == 1:
            print(f"------------{Fore.CYAN}Response Headers{Style.RESET_ALL}-----------")
        for key in headers_dict.keys():
            print(f"{Fore.LIGHTMAGENTA_EX}{key}{Style.RESET_ALL}: {headers_dict[key]}")

        print("-"*39+"\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys

from spyderml.lib import spyder_reqs
from spyderml.lib.asciiarts import banner
from spyderml.lib.handle_headers_and_proxy import handle_headers, handle_proxy, handle_data
from spyderml.lib.spidercrawler import get_all_urls, WebCrawling
from spyderml.lib.utils import treat_objects, soup_tags, soup_comments, soup_attrs, get_js, \
    detect_technologies, update_database, print_html, json_response, print_headers
from spyderml.lib.file import open_file

sys.stdin.reconfigure(encoding='utf-8')
sys.stdout.reconfigure(encoding='utf-8')


class TreatArguments:
    def __init__(self, args):
        self.args = args

    def run(self):
        print(banner())
        headers = handle_headers(useragent=self.args['agent'],
                                 cookies=self.args['cookie'],
                                 headersfile=self.args['headersfile'])
        proxy = handle_proxy(self.args['proxy'])
        data = handle_data(self.args['data'])
        try:
            if self.args['update']:
                update_database()

            else:
                if headers is not None:
                    spyder_request = spyder_reqs.Cacherequest(life=60, cache=self.args['cache'],
                                                              proxy=proxy,
                                                              headers=headers)
                    if self.args['tr'].lower() == 'post':
                        spyder_request.post(url=self.args['target'], data=data)
                    elif self.args['tr'].lower() == 'get':
                        spyder_request.get(url=self.args['target'])
                    html_document = spyder_request.text
                else:
                    spyder_request = spyder_reqs.Cacherequest(life=60, cache=self.args['cache'],
                                                              proxy=proxy)
                    if self.args['tr'].lower() == 'post':
                        spyder_request.post(url=self.args['target'], data=data)
                    elif self.args['tr'].lower() == 'get':
                        spyder_request.get(url=self.args['target'])
                    html_document = spyder_request.text

                if self.args['tags']:
                    tags = treat_objects(objects=self.args['tags'])
                    soup_tags(document=html_document, object=tags, file=self.args['output'])
                elif self.args['comments']:
                    soup_comments(document=html_document, file=self.args['output'])
                elif self.args['getjs']:
                    get_js(url=self.args['target'], document=html_document, file=self.args['output'])
                elif self.args['techs']:
                    detect_technologies(url=self.args['target'], filepath=self.args['output'])
                elif self.args['geturls']:
                    if self.args['spider']:
                        wc = WebCrawling(permission=self.args['spider'], workers=self.args['workers'],
                                         filepath=self.args['output'], domain=self.args['domain'])
                        wc.pre_queue_to_crawling(url=self.args['target'])
                    else:
                        get_all_urls(document=html_document, file=self.args['output'])
                elif self.args['html']:
                    print_html(document=html_document, file=self.args['output'])
                elif self.args['jsr']:
                    json_response(document=html_document, file=self.args['output'])

                elif self.args['attribs']:
                    attr = treat_objects(objects=self.args['attribs'])
                    soup_attrs(document=html_document, object=attr, file=self.args['output'])
                else:
                    spyder_request.response_and_request_headers(url=self.args['target'])
                    print_headers(spyder_request.headers_text, filenane=self.args['output'])
        except KeyboardInterrupt:
            print("\nClossing...\n")
            sys.exit(0)


def main():
    parser = argparse.ArgumentParser(description="A tool made to facilitate the analysis of html code.")

    group_target = parser.add_mutually_exclusive_group()

    group_action = parser.add_mutually_exclusive_group()

    group_target.add_argument('-t', '--target', type=str, help="Parameter that defines the target URL "
                                                               "http://example.com/index.html")
    parser.add_argument('--tr', type=str, default="GET",
                        help='Type of request.')
    group_target.add_argument('--update', help='Flag responsible for updating the database.',
                              default=False, action='store_true')
    group_action.add_argument('--tags', type=str, help="Flag that defines which tags the program will bring")

    group_action.add_argument('--comments', help="Flag that brings the comments", default=False, action='store_true', )

    group_action.add_argument('--attribs', type=str,
                              help="Flag that defines which attributes the application will look for.")

    group_action.add_argument('--getjs', help='Flag that brings all JS files from the page.',
                              default=False, action='store_true')
    group_action.add_argument('--techs', help='Flag trying to discover the technologies of the page.',
                              default=False, action='store_true')
    group_action.add_argument('--geturls', help='This flag takes all the target\'s urls and tries to access them.',
                              default=False, action='store_true')

    group_action.add_argument('--html', help='This Flag results in all the page\'s html code.',
                              default=False, action='store_true')
    group_action.add_argument('--jsr', action='store_true', default=False,
                              help="Makes a request that returns a JSON.")
    parser.add_argument('-o', '--output', type=str,
                        help="Flag that defines in which file the command output will be "
                             "saved.")
    parser.add_argument('-C', '--cookie', type=str, help="Cookie to send with the request",
                        default=None)
    parser.add_argument('-A', '--agent', type=str, help="User-Agent to send with the request",
                        default=None)
    parser.add_argument('-hf', '--headersfile', type=str, default=None,
                        help="Parameter that passes an HTTP request"
                             + " header file to be scanned.")

    parser.add_argument('-S', '--spider', action='store_true',
                        help='flag to run spider', default=False)

    parser.add_argument('-w', '--workers', type=int, default=4,
                        help="Defines the number of workers.")
    parser.add_argument('--domain', type=str, default=None,
                        help="Defines the domain of the web crawler.")
    parser.add_argument('--cache', action='store_true', default=False,
                        help="Defines whether to create cache or not (default: false).")
    parser.add_argument('--proxy', type=str, default=None,
                        help="Defines the proxy that will be used (Which can be passed tor or burpsuite to use these"
                             + " two default proxies).")
    parser.add_argument('-D', '--data', nargs='+',
                        help="Data to send with the request in format key1:value1 key2:value2 key3:value3...")

    args = vars(parser.parse_args())

    if not any(args.values()):
        parser.error('No arguments provided.')
        sys.exit(0)
    if not args['target'] and not args['update']:
        parser.error('No url the file with defined urls.')
        sys.exit(0)
    if args['spider'] and not args['geturls']:
        parser.error('--spider can only be used with --geturls')
        sys.exit(0)
    if args['agent'] and args['cookie'] and args['headersfile']:
        parser.error("The arguments --cookie, --agent --headers file cannot be used together.")
        sys.exit(0)
    if args['agent'] and args['headersfile']:
        parser.error("The arguments --agent, --headers file cannot be used together.")
        sys.exit(0)
    if args['cookie'] and args['headersfile']:
        parser.error("The arguments --cookie, --headers file cannot be used together.")
        sys.exit(0)
    sp = TreatArguments(args)
    sp.run()


if __name__ == '__main__':
    main()

# comentando uma linha de código Ctrl + /



from spyderml.lib.file import open_headers_file


def handle_headers(useragent, cookies, headersfile):
    if useragent is not None or cookies is not None:
        if useragent and cookies:
            headers = {
                "User-Agent": useragent,
                "Cookie": "; ".join(cookies)
            }
        elif cookies:
            headers = {
                'Cookie': "; ".join(cookies)
            }
        else:
            headers = {
                'User-Agent': useragent
            }

    elif headersfile is not None:
        headers = open_headers_file(headers_file=headersfile)
    else:
        headers = None

    return headers


def handle_proxy(proxy):
    if proxy == 'burpsuite':
        proxies = {
            'http': 'http://localhost:8080',
            'https': 'http://localhost:8080'
        }
    elif proxy is None:
        return proxy
    elif proxy == 'tor':
        proxies = {
            'http': 'socks5h://localhost:9050',
            'https': 'socks5h://localhost:9050'
        }
    else:
        proxies = {
            'http': proxy,
            'https': proxy
        }
    return proxies


def handle_data(data):
    if data:
        data_dict = {}
        for item in data:
            key, value = item.split(':')
            data_dict[key] = value
    else:
        data_dict = None
    return data_dict
